package org.xbib.jdbc.connection.pool;

import java.text.MessageFormat;
import java.util.concurrent.Callable;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

import static org.xbib.jdbc.connection.pool.Pool.POOL_NORMAL;

/**
 * Creating and adding poolEntries (connections) to the pool.
 */
public class PoolEntryCreator implements Callable<Boolean> {

    private static final Logger logger = Logger.getLogger(PoolEntryCreator.class.getName());

    private final Pool pool;

    private final String loggingPrefix;

    PoolEntryCreator(Pool pool, String loggingPrefix) {
        this.pool = pool;
        this.loggingPrefix = loggingPrefix;
    }

    @Override
    public Boolean call() {
        long sleepBackoff = 250L;
        while (pool.poolState == POOL_NORMAL && pool.shouldCreateAnotherConnection()) {
            final PoolEntry poolEntry = pool.createPoolEntry();
            if (poolEntry != null) {
                pool.bag.add(poolEntry);
                logger.log(Level.FINE, () -> MessageFormat.format("{0}: added connection {1} ",
                        pool.poolName, poolEntry.getConnection()));
                if (loggingPrefix != null) {
                    pool.logPoolState(loggingPrefix);
                }
                return Boolean.TRUE;
            }
            if (loggingPrefix != null) {
                logger.log(Level.FINE, () -> "connection add failed, sleeping with backoff" + pool.poolName);
            }
            Pool.quietlySleep(sleepBackoff);
            sleepBackoff = Math.min(TimeUnit.SECONDS.toMillis(10), Math.min(pool.connectionTimeout, (long) (sleepBackoff * 1.5)));
        }
        return Boolean.FALSE;
    }
}
