package org.xbib.jdbc.oracle.test;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.testcontainers.containers.OracleContainer;
import org.testcontainers.junit.jupiter.Testcontainers;
import org.xbib.jdbc.query.Config;
import org.xbib.jdbc.query.ConfigSupplier;
import org.xbib.jdbc.query.DatabaseProvider;
import org.xbib.jdbc.query.OptionsOverride;
import org.xbib.jdbc.query.Schema;
import org.xbib.jdbc.test.CommonTest;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

/**
 * Exercise Database functionality with a real Oracle database.
 */
@Testcontainers
public class OracleTest extends CommonTest {

    static OracleContainer oracleContainer;

    static {
        oracleContainer = new OracleContainer("gvenzl/oracle-xe:21.3.0-slim")
                .withDatabaseName("testDB")
                .withUsername("testUser")
                .withPassword("testPassword")
                .withExposedPorts(1521)
                .withReuse(true);
    }

    @BeforeAll
    public static void startContainer() {
        oracleContainer.start();
    }

    @AfterAll
    public static void stopContainer() {
        oracleContainer.stop();
    }

    @Override
    protected DatabaseProvider createDatabaseProvider(OptionsOverride options) throws Exception {
        Config config = ConfigSupplier.of()
                .property("database.url", oracleContainer.getJdbcUrl())
                .property("database.user", "testUser")
                .property("database.password", "testPassword")
                .get();
        return DatabaseProvider.builder(config)
                .withSqlParameterLogging()
                .withSqlInExceptionMessages()
                .withOptions(options)
                .build();
    }

    @Disabled("Current Oracle behavior is to convert -0f to 0f")
    @Test
    public void argFloatNegativeZero() {
        super.argFloatNegativeZero();
    }

    @Disabled("Current Oracle behavior is to convert -0d to 0d")
    @Test
    public void argDoubleNegativeZero() {
        super.argDoubleNegativeZero();
    }

    @Override
    @Test
    public void metadataColumnNames() {
        db.dropTableQuietly("dbtest");
        new Schema().addTable("dbtest").addColumn("pk").primaryKey().schema().execute(db);
        db.toSelect("select Pk, Pk as Foo, Pk as \"Foo\" from dbtest")
                .query(rs -> {
                    Assertions.assertArrayEquals(new String[]{"PK", "FOO", "Foo"}, rs.getColumnLabels());
                    return null;
                });
    }

    @Test
    public void testTableExists() {
        db.dropTableQuietly("dbtest");
        new Schema().addTable("dbtest").addColumn("pk").primaryKey().schema().execute(db);
        assertTrue(db.tableExists("dbtest"));
        db.dropTableQuietly("dbtest");
        assertFalse(db.tableExists("dbtest"));
        db.ddl("create table \"dbtest\" (pk varchar(1))").execute();
        assertTrue(db.tableExists("dbtest"));
        db.dropTableQuietly("dbtest");
    }
}
