package org.xbib.jdbc.postgresql;

import org.postgresql.PGNotification;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.function.Consumer;

public class Listener implements Runnable {

    private final Connection connection;

    private final org.postgresql.PGConnection pgConnection;

    private final Consumer<PGNotification> notificationConsumer;

    private final Consumer<Throwable> errorConsumer;

    public Listener(Connection connection,
                    Consumer<PGNotification> notificationConsumer,
                    Consumer<Throwable> errorConsumer) throws SQLException {
        this.connection = connection;
        this.pgConnection = (org.postgresql.PGConnection) connection;
        this.errorConsumer = errorConsumer;
        this.notificationConsumer = notificationConsumer;
        try (Statement stmt = connection.createStatement()) {
            stmt.execute("LISTEN event");
        }
    }

    @Override
    public void run() {
        while (true) {
            try (Statement stmt = connection.createStatement();
                 ResultSet rs = stmt.executeQuery("SELECT 1")) {
                //
            } catch (SQLException e) {
                errorConsumer.accept(e);
            }
            try {
                PGNotification[] notifications = pgConnection.getNotifications();
                if (notifications != null) {
                    for (PGNotification notification : notifications) {
                        notificationConsumer.accept(notification);
                    }
                }
            } catch (SQLException e) {
                errorConsumer.accept(e);
            }
        }
    }
}
