package org.xbib.jdbc.query;

import org.xbib.jdbc.query.flavor.Derby;

/**
 * Base class for selectively overriding another Options object.
 */
public class OptionsOverride implements Options {

    private Options parent;

    /**
     * Wrap another {@code Options} and defer to it for anything we choose not
     * to override.
     */
    public OptionsOverride(Options parent) {
        this.parent = parent;
    }

    /**
     * Defer to OptionsDefault for anything that is not specified.
     */
    public OptionsOverride() {
        parent = new OptionsDefault(new Derby());
    }

    /**
     * Defer to OptionsDefault for anything that is not specified, using the specified flavor.
     */
    public OptionsOverride(Flavor flavor) {
        parent = new OptionsDefault(flavor);
    }

    public void setParent(Options parent) {
        this.parent = parent;
    }

    public OptionsOverride withParent(Options parent) {
        this.parent = parent;
        return this;
    }

    @Override
    public int fetchSize() {
        return parent.fetchSize();
    }

    @Override
    public int timeoutSeconds() {
        return parent.timeoutSeconds();
    }

    @Override
    public boolean allowTransactionControl() {
        return parent.allowTransactionControl();
    }

    @Override
    public boolean ignoreTransactionControl() {
        return parent.ignoreTransactionControl();
    }

    @Override
    public boolean allowConnectionAccess() {
        return parent.allowConnectionAccess();
    }

    @Override
    public boolean isLogParameters() {
        return parent.isLogParameters();
    }

    @Override
    public boolean isDetailedExceptions() {
        return parent.isDetailedExceptions();
    }

    @Override
    public String generateErrorCode() {
        return parent.generateErrorCode();
    }

    @Override
    public boolean useBytesForBlob() {
        return parent.useBytesForBlob();
    }

    @Override
    public boolean useStringForClob() {
        return parent.useStringForClob();
    }

    @Override
    public Flavor flavor() {
        return parent.flavor();
    }

    @Override
    public boolean useLocalDateTimeOnly() {
        return parent.useLocalDateTimeOnly();
    }

    @Override
    public int maxStringLengthParam() {
        return parent.maxStringLengthParam();
    }
}
