package org.xbib.jdbc.query.flavor;

import org.xbib.jdbc.query.Flavor;

import java.sql.PreparedStatement;
import java.sql.SQLException;

public class H2 implements Flavor {

    @Override
    public String getName() {
        return "h2";
    }

    @Override
    public boolean supports(String url) {
        return url.startsWith("jdbc:h2:");
    }

    @Override
    public String driverClass() {
        return "org.h2.Driver";
    }

    @Override
    public String normalizeTableName(String tableName) {
        if (tableName == null) {
            return tableName;
        }
        if (tableName.length() > 2) {
            if (tableName.startsWith("\"") && tableName.endsWith("\"")) {
                return tableName.substring(1, tableName.length() - 1);
            }
        }
        return tableName.toUpperCase();
    }

    @Override
    public String typeInteger() {
        return "integer";
    }

    @Override
    public String typeBoolean() {
        return "char(1)";
    }

    @Override
    public String typeLong() {
        return "bigint";
    }

    @Override
    public String typeFloat() {
        return "double";
    }

    @Override
    public void setFloat(PreparedStatement preparedStatement, int i, Float floatValue) throws SQLException {
        preparedStatement.setFloat(i, floatValue);
    }

    @Override
    public String typeDouble() {
        return "double";
    }

    @Override
    public void setDouble(PreparedStatement preparedStatement, int i, Double doubleValue) throws SQLException {
        preparedStatement.setDouble(i, doubleValue);
    }

    @Override
    public String typeBigDecimal(int size, int precision) {
        return "numeric(" + size + "," + precision + ")";
    }

    @Override
    public String typeStringVar(int length) {
        return "varchar(" + length + ")";
    }

    @Override
    public String typeStringFixed(int length) {
        return "char(" + length + ")";
    }

    @Override
    public String typeClob() {
        return "clob(2G)";
    }

    @Override
    public String typeBlob() {
        return "blob(2G)";
    }

    @Override
    public String typeLocalDateTime() {
        return "timestamp(3)";
    }

    @Override
    public String columnTypeLocalDateTime() {
        return "timestamp";
    }

    @Override
    public String typeLocalDate() {
        return "date";
    }

    @Override
    public boolean useStringForClob() {
        return true;
    }

    @Override
    public boolean useBytesForBlob() {
        return true;
    }

    @Override
    public String sequenceNextVal(String sequenceName) {
        return "next value for " + sequenceName + "";
    }

    @Override
    public String sequenceSelectNextVal(String sequenceName) {
        return "select " + sequenceNextVal(sequenceName) + fromAny();
    }

    @Override
    public String sequenceDrop(String dbtestSeq) {
        return "drop sequence if exists " + dbtestSeq;
    }

    @Override
    public String tableDrop(String table) {
        return "drop table if exists " + table;
    }

    @Override
    public String sequenceOrderClause(boolean order) {
        return "";
    }

    @Override
    public String sequenceCycleClause(boolean cycle) {
        return cycle ? " cycle" : " no cycle";
    }

    @Override
    public String fromAny() {
        return " from (values(0))";
    }

    @Override
    public boolean supportsInsertReturning() {
        return false;
    }

    @Override
    public String dbTimeMillis() {
        return "localtimestamp";
    }

    @Override
    public String sequenceCacheClause(int nbrValuesToCache) {
        return "";
    }

    @Override
    public String sequenceOptions() {
        return " as bigint";
    }

    @Override
    public boolean isAutoCommitOnly() {
        return false;
    }

    @Override
    public String queueLength(String table) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String writeNextIntoQueue(String table) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String readNextFromQueue(String table) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String succeedInQueue(String table) {
        throw new UnsupportedOperationException();
    }
    @Override
    public String failInQueue(String table) {
        throw new UnsupportedOperationException();
    }
}
