package org.xbib.jdbc.sqlserver;

import org.xbib.jdbc.query.Flavor;

import java.sql.PreparedStatement;
import java.sql.SQLException;

public class SqlServer implements Flavor {

    public SqlServer() {
    }

    @Override
    public String getName() {
        return "sqlServer";
    }

    @Override
    public boolean supports(String url) {
        return url.startsWith("jdbc:sqlserver:");
    }

    @Override
    public String driverClass() {
        return "com.microsoft.sqlserver.jdbc.SQLServerDriver";
    }

    @Override
    public String normalizeTableName(String tableName) {
        if (tableName == null) {
            return tableName;
        }
        if (tableName.length() > 2) {
            if (tableName.startsWith("\"") && tableName.endsWith("\"")) {
                return tableName.substring(1, tableName.length() - 1);
            }
        }
        return tableName;
    }

    @Override
    public String typeFloat() {
        return "float(24)";
    }

    @Override
    public void setFloat(PreparedStatement preparedStatement, int i, Float floatValue) throws SQLException {
        preparedStatement.setFloat(i, floatValue);
    }

    @Override
    public String typeDouble() {
        return "float(53)";
    }

    @Override
    public void setDouble(PreparedStatement preparedStatement, int i, Double doubleValue) throws SQLException {
        preparedStatement.setDouble(i, doubleValue);
    }

    @Override
    public String typeBigDecimal(int size, int precision) {
        return "numeric(" + size + "," + precision + ")";
    }

    @Override
    public String typeInteger() {
        return "numeric(10)";
    }

    @Override
    public String typeBoolean() {
        return "char(1)";
    }

    @Override
    public String typeLong() {
        return "numeric(19)";
    }

    @Override
    public String typeLocalDateTime() {
        return "datetime2(3)";
    }

    @Override
    public String columnTypeLocalDateTime() {
        return "datetime2";
    }

    @Override
    public String typeLocalDate() {
        return "date";
    }

    @Override
    public boolean useStringForClob() {
        return false;
    }

    @Override
    public boolean useBytesForBlob() {
        return false;
    }

    @Override
    public String sequenceNextVal(String sequenceName) {
        return "next value for " + sequenceName;
    }

    @Override
    public String sequenceSelectNextVal(String sequenceName) {
        return "select next value for " + sequenceName;
    }

    @Override
    public String sequenceDrop(String dbtestSeq) {
        return "drop sequence " + dbtestSeq;
    }

    @Override
    public String tableDrop(String table) {
        return "drop table " + table;
    }

    @Override
    public String typeStringVar(int length) {
        return "varchar(" + length + ")";
    }

    @Override
    public String typeStringFixed(int length) {
        return "char(" + length + ")";
    }

    @Override
    public String typeClob() {
        return "varchar(max)";
    }

    @Override
    public String typeBlob() {
        return "varbinary(max)";
    }

    @Override
    public String sequenceOrderClause(boolean order) {
        // Not supported
        return "";
    }

    @Override
    public String sequenceCycleClause(boolean cycle) {
        return cycle ? " cycle" : " no cycle";
    }

    @Override
    public boolean supportsInsertReturning() {
        return true;
    }

    @Override
    public String dbTimeMillis() {
        return "current_timestamp";
    }

    @Override
    public String sequenceCacheClause(int nbrValuesToCache) {
        if (nbrValuesToCache < 2) {
            return " no cache";
        }
        return " cache " + nbrValuesToCache;
    }

    @Override
    public String fromAny() {
        return "";
    }

    @Override
    public String sequenceOptions() {
        return "";
    }

    @Override
    public boolean isAutoCommitOnly() {
        return false;
    }

    @Override
    public String queueLength(String table) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String writeNextIntoQueue(String table) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String readNextFromQueue(String table) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String succeedInQueue(String table) {
        throw new UnsupportedOperationException();
    }

    @Override
    public String failInQueue(String table) {
        throw new UnsupportedOperationException();
    }
}
